# -*- coding: utf-8 -*-

"""代码写到一定的量之后发现，实在是太多的字符串常量了。另人难过的差不多的语境，不同时期的拼写还不一致。
作者: 蒋乐兴|neeky
时间: 2023-03
"""

# 函数开执行
FUN_STARTS = "starts {} ."
# 函数执行完成(return 之前)
FUN_ENDS = "ends {} ."
VIEW_FUN_STARTS = "view-request-starts: {}"
VIEW_FUN_ENDS = "view-requests-ends: {}"


# 文件不存在
FILE_NOT_EXISTS = "file '{}' not exists ."


# 执行命令
EXECUTE_CMD = "excute cmd '{}' ."


# 创建目录
CREATE_DIR = "create dir '{}' ."
DIR_EXISTS = "dir '{}' exists ."
DIR_NOT_EXISTS = "dir '{}' not exists ."
MOVE_FILE_TO = "move '{}' to '{}' "


# MySQL MESSAGES
MYSQL_INSTANCE_HAS_EXISTS = "mysql instance '{}' has exists ."


ARG_NOT_IN_POST_DICT = "argument '{}' not in post dict"
